@@ -34,9 +34,9 @@ class ExploitUtil:
         """
         # Extract target machine
         target_machine = EmulationAttackerMachineObservationState(ips=a.ips)
-        total_time = 0
+        total_time = 0.0
         exploit_successful = False
-        cost = 0
+        cost = 0.0
         for ip in target_machine.ips:
             try:
                 jump_connection = ConnectionUtil.find_jump_host_connection(ip=ip, s=s)
@@ -46,6 +46,8 @@ class ExploitUtil:
                     cmd = cmd.format(ip)
                     outdata, errdata, total_time = EmulationUtil.execute_ssh_cmd(
                         cmd=cmd, conn=jump_connection.conn)
+                    Logger.__call__().get_logger().warning(
+                        f"Samba command: {cmd}, out: {outdata.decode()}, err: {errdata.decode()}")
 
                     # Parse result and check outcome
                     exploit_successful = (constants.SAMBA.ALREADY_EXISTS in outdata.decode() or
@@ -66,6 +68,9 @@ class ExploitUtil:
         # Parse Result
         if exploit_successful:
 
+            Logger.__call__().get_logger().info(f"The SambaCry exploit against machine: {target_machine.ips[0]} "
+                                                f"was successful, cmd: {a.cmds[0].format(target_machine.ips[0])}")
+
             # Exploit successful
             credential = Credential(username=constants.SAMBA.BACKDOOR_USER, pw=constants.SAMBA.BACKDOOR_PW,
                                     port=constants.SSH.DEFAULT_PORT, protocol=TransportProtocol.TCP,
@@ -91,6 +96,9 @@ class ExploitUtil:
 
             EmulationUtil.log_measured_action_time(total_time=total_time, action=a,
                                                    emulation_env_config=s.emulation_env_config)
+        else:
+            Logger.__call__().get_logger().info(f"The SambaCry exploit against machine: {target_machine.ips[0]} "
+                                                f"was unsuccessful")
 
         target_machine.sambacry_tried = True
         attacker_machine_observations = EnvDynamicsUtil.merge_new_obs_with_old(
@@ -111,8 +119,8 @@ class ExploitUtil:
         """
         target_machine = EmulationAttackerMachineObservationState(ips=a.ips)
         exploit_successful = False
-        cost = 0
-        total_time = 0
+        cost = 0.0
+        total_time = 0.0
         for ip in target_machine.ips:
             try:
                 jump_connection = ConnectionUtil.find_jump_host_connection(ip=ip, s=s)
@@ -124,7 +132,8 @@ class ExploitUtil:
                                      constants.SHELLSHOCK.BACKDOOR_USER, ip)
                     outdata, errdata, total_time = EmulationUtil.execute_ssh_cmd(
                         cmd=cmd, conn=jump_connection.conn)
-
+                    Logger.__call__().get_logger().info(f"Shellshock cmd: {cmd}, out: {outdata.decode()}, "
+                                                        f"err:{errdata.decode()}")
                     cmd = a.cmds[1]
                     cmd = cmd.format(constants.SHELLSHOCK.BACKDOOR_USER, constants.SHELLSHOCK.BACKDOOR_PW, a.ips)
                     outdata1, errdata1, total_time1 = EmulationUtil.execute_ssh_cmd(
@@ -150,9 +159,11 @@ class ExploitUtil:
             except Exception as e:
                 Logger.__call__().get_logger().warning(f"There was an error executing the shellshock exploit: "
                                                        f"{str(e)}, {repr(e)}")
-
         if exploit_successful:
 
+            Logger.__call__().get_logger().info(f"The Shellshock exploit against machine: {target_machine.ips[0]} "
+                                                f"was successful")
+
             # Exploit successful
             credential = Credential(username=constants.SHELLSHOCK.BACKDOOR_USER,
                                     pw=constants.SHELLSHOCK.BACKDOOR_PW,
@@ -174,6 +185,9 @@ class ExploitUtil:
             target_machine.backdoor_tried = True
             target_machine.backdoor_installed = True
             target_machine.cve_vulns.append(vuln)
+        else:
+            Logger.__call__().get_logger().info(f"The Shellshock exploit against machine: {target_machine.ips[0]} "
+                                                f"was unsuccessful")
 
         # Measure  cost and alerts
         cost += float(total_time)
@@ -212,7 +226,9 @@ class ExploitUtil:
             target_conn = paramiko.SSHClient()
             target_conn.set_missing_host_key_policy(paramiko.AutoAddPolicy())
             target_conn.connect(target_ip, username=user, password=pw, sock=relay_channel)
-            target_conn.get_transport().set_keepalive(5)
+            transport = target_conn.get_transport()
+            if transport is not None:
+                transport.set_keepalive(5)
             return True
         except Exception as e:
             Logger.__call__().get_logger().debug(
@@ -229,10 +245,9 @@ class ExploitUtil:
         :return: s_prime
         """
         target_machine = EmulationAttackerMachineObservationState(ips=a.ips)
-        cost = 0
-        total_time = 0
+        cost = 0.0
+        total_time = 0.0
         exploit_successful = False
-        pw = "-"
         for ip in target_machine.ips:
             try:
                 jump_connection = ConnectionUtil.find_jump_host_connection(ip=ip, s=s)
@@ -241,6 +256,8 @@ class ExploitUtil:
                     cmd = cmd + " " + ip
                     outdata, errdata, total_time = EmulationUtil.execute_ssh_cmd(
                         cmd=cmd, conn=jump_connection.conn)
+                    Logger.__call__().get_logger().info(f"Sql injeciton cmd: {cmd}, out: {outdata.decode()}, "
+                                                        f"err:{errdata.decode()}")
 
                     # Parse Result
                     exploit_successful = False
@@ -263,6 +280,8 @@ class ExploitUtil:
                         pw = "-"
 
                     if exploit_successful:
+                        Logger.__call__().get_logger().info(f"SQL injection against: {target_machine.ips[0]} "
+                                                            f"was successful")
                         ExploitUtil.remove_dvwa_sql_injection_result(conn=jump_connection.conn,
                                                                      dir=dir)
                         # Exploit successful
@@ -291,6 +310,8 @@ class ExploitUtil:
                         cost += float(total_time)
                         break
                     else:
+                        Logger.__call__().get_logger().info(f"SQL injection against: {target_machine.ips[0]} "
+                                                            f"was not successful")
                         time.sleep(constants.ENV_CONSTANTS.ATTACKER_RETRY_DVWA_SQL_INJECTION)
 
                 if exploit_successful:
@@ -321,13 +342,12 @@ class ExploitUtil:
         """
         sftp_client = conn.open_sftp()
         remote_file = sftp_client.open(dir + constants.DVWA_SQL_INJECTION.EXPLOIT_OUTPUT_FILENAME)
-        result = None
         try:
             data = remote_file.read()
             result = data.decode()
         finally:
             remote_file.close()
-        return result
+        return str(result)
 
     @staticmethod
     def remove_dvwa_sql_injection_result(conn, dir: str) -> None:
@@ -352,8 +372,8 @@ class ExploitUtil:
         # Extract target machine
         target_machine = EmulationAttackerMachineObservationState(ips=a.ips)
         exploit_successful = False
-        cost = 0
-        total_time = 0
+        cost = 0.0
+        total_time = 0.0
 
         for ip in target_machine.ips:
             try:
@@ -364,7 +384,8 @@ class ExploitUtil:
                     cmd = cmd.format(a.ips)
                     outdata, errdata, total_time = EmulationUtil.execute_ssh_cmd(
                         cmd=cmd, conn=jump_connection.conn)
-
+                    Logger.__call__().get_logger().info(f"CVE-2015-3306, cmd: {cmd}, out: {outdata.decode()}, "
+                                                        f"err: {errdata.decode()}")
                     proxy_conn = EmulationConnectionObservationState(
                         conn=jump_connection.conn, credential=jump_connection.credential,
                         root=True, port=constants.SSH.DEFAULT_PORT, service=constants.SSH.SERVICE_NAME,
@@ -386,7 +407,8 @@ class ExploitUtil:
                     f"There was an error executing the CVE-2015-3306 exploit: {e}, {repr(e)}")
 
         if exploit_successful:
-
+            Logger.__call__().get_logger().info(f"CVE-2015-3306 against target: {target_machine.ips[0]} "
+                                                f"was successful")
             # Exploit successful
             credential = Credential(username=constants.CVE_2015_3306.BACKDOOR_USER,
                                     pw=constants.CVE_2015_3306.BACKDOOR_PW,
@@ -408,6 +430,9 @@ class ExploitUtil:
             target_machine.backdoor_tried = True
             target_machine.backdoor_installed = True
             target_machine.cve_vulns.append(vuln)
+        else:
+            Logger.__call__().get_logger().info(f"CVE-2015-3306 against target: {target_machine.ips[0]} "
+                                                f"was not successful")
 
         # Measure  cost and alerts
         cost += float(total_time)
@@ -434,8 +459,8 @@ class ExploitUtil:
         """
         # Extract target machine
         target_machine = EmulationAttackerMachineObservationState(ips=a.ips)
-        cost = 0
-        total_time = 0
+        cost = 0.0
+        total_time = 0.0
         exploit_successful = False
         for ip in target_machine.ips:
             try:
@@ -446,6 +471,8 @@ class ExploitUtil:
                     cmd = cmd.format(a.ips)
                     outdata, errdata, total_time = EmulationUtil.execute_ssh_cmd(
                         cmd=cmd, conn=jump_connection.conn)
+                    Logger.__call__().get_logger().info(f"CVE-2015-1427 cmd:{cmd}, out:{outdata.decode()}, "
+                                                        f"err: {errdata.decode()}")
 
                     # Parse Result
                     proxy_conn = EmulationConnectionObservationState(
@@ -469,6 +496,8 @@ class ExploitUtil:
                     f"There was an error executing the CVE-2015-1427 exploit: {e}, {repr(e)}")
 
         if exploit_successful:
+            Logger.__call__().get_logger().info(f"CVE-2015-1427 against target: {target_machine.ips[0]} was successful")
+
             # Exploit successful
             credential = Credential(username=constants.CVE_2015_1427.BACKDOOR_USER,
                                     pw=constants.CVE_2015_1427.BACKDOOR_PW,
@@ -490,6 +519,9 @@ class ExploitUtil:
             target_machine.backdoor_tried = True
             target_machine.backdoor_installed = True
             target_machine.cve_vulns.append(vuln)
+        else:
+            Logger.__call__().get_logger().info(f"CVE-2015-1427 against target: {target_machine.ips[0]} "
+                                                f"was not successful")
 
         # Measure  cost and alerts
         cost += float(total_time)
@@ -515,8 +547,8 @@ class ExploitUtil:
         """
         # Extract target machine
         target_machine = EmulationAttackerMachineObservationState(ips=a.ips)
-        cost = 0
-        total_time = 0
+        cost = 0.0
+        total_time = 0.0
         exploit_successful = False
         for ip in target_machine.ips:
             try:
@@ -528,6 +560,8 @@ class ExploitUtil:
                     cmd = cmd.format(a.ips)
                     outdata, errdata, total_time = EmulationUtil.execute_ssh_cmd(
                         cmd=cmd, conn=jump_connection.conn)
+                    Logger.__call__().get_logger().info(f"CVE-2016-10033 cmd:{cmd}, out: {outdata.decode()}, "
+                                                        f"err: {errdata.decode()}")
 
                     # Parse Result
                     proxy_conn = EmulationConnectionObservationState(
@@ -551,6 +585,8 @@ class ExploitUtil:
                     f"There was an exception executing exploit CVE-2016-10033: {e}, {repr(e)}")
 
         if exploit_successful:
+            Logger.__call__().get_logger().info(f"CVE-2016-10033 against {target_machine.ips[0]} was succcessful")
+
             # Exploit successful
             credential = Credential(username=constants.CVE_2016_10033.BACKDOOR_USER,
                                     pw=constants.CVE_2016_10033.BACKDOOR_PW,
@@ -572,6 +608,8 @@ class ExploitUtil:
             target_machine.backdoor_tried = True
             target_machine.backdoor_installed = True
             target_machine.cve_vulns.append(vuln)
+        else:
+            Logger.__call__().get_logger().info(f"CVE-2016-10033 against {target_machine.ips[0]} was not succcessful")
 
         # Measure  cost and alerts
         cost += float(total_time)
@@ -600,10 +638,10 @@ class ExploitUtil:
         :param a: the CVE-2010-0425 privilege escalation action
         :return: new_machine_obs, cost, successful, root_credential, service_name
         """
-        cost = 0
+        cost = 0.0
         e_succ = False
         root_credential = None
-        service = None
+        service = ""
         sleep_time = 0.5
 
         # Start with SSH connections
@@ -643,6 +681,7 @@ class ExploitUtil:
                             e_succ = False
 
                         if e_succ:
+                            Logger.__call__().get_logger().info("CVE-2010-0426 was succcessful")
                             root_credential = Credential(
                                 username=constants.CVE_2010_0426.BACKDOOR_USER,
                                 pw=constants.CVE_2010_0426.BACKDOOR_PW, port=constants.SSH.DEFAULT_PORT,
@@ -671,6 +710,7 @@ class ExploitUtil:
                             service = constants.SSH.SERVICE_NAME
                             break
                         else:
+                            Logger.__call__().get_logger().info("CVE-2010-0426 against was not succcessful")
                             time.sleep(constants.ENV_CONSTANTS.ATTACKER_CVE_2010_0426_SLEEP_RETRY)
                     if e_succ:
                         break
@@ -696,10 +736,10 @@ class ExploitUtil:
         :param a: the CVE-2015-5602 privilege escalation action
         :return: s_prime, cost, e_succ (flag), credentials, servicename
         """
-        cost = 0
+        cost = 0.0
         e_succ = False
         root_credential = None
-        service = None
+        service = ""
         sleep_time = 0.5
 
         # Start with SSH connections
@@ -751,6 +791,7 @@ class ExploitUtil:
                         e_succ = False
 
                     if e_succ:
+                        Logger.__call__().get_logger().info("CVE-2015-5602 was succcessful")
                         root_credential = Credential(username=constants.CVE_2015_5602.BACKDOOR_USER,
                                                      pw=constants.CVE_2015_5602.BACKDOOR_PW,
                                                      port=constants.SSH.DEFAULT_PORT,
