@@ -1,6 +1,6 @@
+from typing import Tuple, List, Union, Any
 import time
 import threading
-from typing import Tuple, List, Union
 import xml.etree.ElementTree as ET
 from csle_common.dao.emulation_config.emulation_env_config import EmulationEnvConfig
 from csle_common.dao.emulation_config.emulation_env_state import EmulationEnvState
@@ -36,12 +36,14 @@ class NmapUtil:
     """
 
     @staticmethod
-    def parse_nmap_scan(file_name: str, emulation_env_config: EmulationEnvConfig, conn=None, dir: str = None) \
+    def parse_nmap_scan(file_name: str, emulation_env_config: EmulationEnvConfig, conn=None, dir: str = "") \
             -> ET.Element:
         """
         Parses an XML file containing the result of an nmap scan
 
         :param file_name: name of the file to parse
+        :param conn: the SSH connection to use for parsing
+        :param dir: the directory to parse the XML file
         :param emulation_env_config: environment config
         :return: the parsed xml file
         """
@@ -70,7 +72,7 @@ class NmapUtil:
         :param action: the action of the scan
         :return: parsed nmap scan result
         """
-        hosts = []
+        hosts: List[NmapHostResult] = []
         for child in xml_data:
             if child.tag == constants.NMAP_XML.HOST:
                 host = NmapUtil._parse_nmap_host_xml(child, action=action)
@@ -90,9 +92,9 @@ class NmapUtil:
         ip_addr = None
         mac_addr = None
         hostnames = []
-        ports = []
-        vulnerabilities = []
-        credentials = []
+        ports: List[NmapPort] = []
+        vulnerabilities: List[NmapVuln] = []
+        credentials: List[NmapBruteCredentials] = []
         os = None
         os_matches = []
         status = NmapHostStatus.UP
@@ -164,8 +166,8 @@ class NmapUtil:
         return hostnames
 
     @staticmethod
-    def _parse_nmap_ports_xml(xml_data, action: EmulationAttackerAction) -> Tuple[List[NmapPort], List[NmapVuln],
-                                                                                  List[NmapBruteCredentials]]:
+    def _parse_nmap_ports_xml(xml_data, action: EmulationAttackerAction) \
+            -> Tuple[List[NmapPort], List[NmapVuln], List[NmapBruteCredentials]]:
         """
         Parses a ports XML element in the XML tree
 
@@ -219,7 +221,7 @@ class NmapUtil:
         return ports, vulnerabilities, credentials
 
     @staticmethod
-    def _parse_nmap_service_name_xml(xml_data) -> str:
+    def _parse_nmap_service_name_xml(xml_data) -> Union[str, Any]:
         """
         Parses a XML service name element
 
@@ -320,10 +322,9 @@ class NmapUtil:
 
     @staticmethod
     def _parse_nmap_script(xml_data, port: int, protocol: TransportProtocol, service: str,
-                           action: EmulationAttackerAction) -> Tuple[Union[List[NmapVuln],
-                                                                           List[NmapBruteCredentials],
-                                                                           NmapHttpEnum, NmapHttpGrep,
-                                                                           NmapVulscan, None], Union[NmapVuln, None]]:
+                           action: EmulationAttackerAction) \
+            -> Tuple[Union[List[NmapVuln], List[NmapBruteCredentials], NmapHttpEnum, NmapHttpGrep, NmapVulscan, None],
+                     Union[NmapVuln, None]]:
         """
         Parses a XML script element
 
@@ -472,6 +473,9 @@ class NmapUtil:
         if masscan:
             cmds = a.masscan_cmds()
 
+        Logger.__call__().get_logger().info(
+            f"Running NMAP scan on container: {s.emulation_env_config.containers_config.agent_ip}, commands: "
+            f"{','.join(cmds)}")
         results = EmulationUtil.execute_ssh_cmds(cmds=cmds, conn=s.emulation_env_config.get_hacker_connection())
         total_time = sum(list(map(lambda x: x[2], results)))
 
@@ -639,7 +643,7 @@ class NmapUtil:
         :return: the new state
         """
         merged_scan_result = partial_result
-        total_results = []
+        total_results: List[NmapScanResult] = []
         threads = []
         for machine in s.attacker_obs_state.machines:
             if machine.logged_in and machine.tools_installed and machine.backdoor_installed:
@@ -686,17 +690,32 @@ class NmapUtil:
 
 
 class PivotNMAPScanThread(threading.Thread):
+    """
+    Thread for asynchronous nmap scanning
+    """
 
     def __init__(self, machine: EmulationAttackerMachineObservationState, a: EmulationAttackerAction,
                  s: EmulationEnvState):
+        """
+        Initializes the thread
+
+        :param machine: the machine to do the details on
+        :param a: the scanning action
+        :param s: the current environment state
+        """
         threading.Thread.__init__(self)
         self.machine = machine
         self.a = a
         self.s = s
         self.total_time = 0
-        self.scan_result = None
+        self.scan_result: NmapScanResult = NmapScanResult(hosts=[], ips=[])
 
     def run(self) -> None:
+        """
+        Main loop of the thread; performs the scanning
+
+        :return: None
+        """
         ssh_connections_alive = []
         for c in self.machine.ssh_connections:
             try:
@@ -719,6 +738,9 @@ class PivotNMAPScanThread(threading.Thread):
             c2 = c
             for i in range(constants.ENV_CONSTANTS.NUM_RETRIES):
                 try:
+                    Logger.__call__().get_logger().info(
+                        f"Running NMAP scan on container(s): {self.machine.ips}, commands: "
+                        f"{','.join(cmds)}")
                     results = EmulationUtil.execute_ssh_cmds(cmds=cmds, conn=c2.conn)
                     break
                 except Exception as e:
